# Copyright 2012 Mail Bypass, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


dir = File.dirname(__FILE__)
require "#{dir}/../spec_helper"

class MessagebusTest < MessagebusApnApi::Messagebus
  attr_accessor :last_init_time
end

describe MessagebusApnApi::Messagebus do
  attr_reader :client, :api_key, :required_params



  def default_apn_send_params
    {
     :userName => 'mark@example.com',
     :alert => 'Message Bus application push',
     :platform => 'ios',
     :customFields => ["badge"=>"23"]
    }
  end

  def default_apn_register_params
    {
     :userName => 'mark@messagebus.com',
     :deviceToken  => '835012e8d4640406818e8c7',
     :platform => 'ios'
    }
  end

  def create_success_result(num_result)
    list=[]
    num_result.times do
      list << @success_message
    end

    success_result = {
        "statusMessage" => "OK",
        "successCount" => num_result,
        "failureCount" => 0,
        "results" => list
    }
    success_result
  end

  def create_results_array
    results = {
        "statusMessage" => "OK",
        "results" => []
    }
    results
  end

  def json_parse(data)
    JSON.parse(data, :symbolize_names => true)
  end

  before do
    FakeWeb.allow_net_connect = false

    @app_key = 'test_key'
    @api_key = "7215ee9c7d9dc229d2921a40e899ec5f"
    @client = MessagebusTest.new(@api_key)
    @success_message={
        "status" => 200,
        "messageId" => "abcdefghijklmnopqrstuvwxyz012345"
    }
    @simple_success_result = create_success_result(1)
    @apn_endpoint = "https://apn.messagebus.com/api/v3/"
  end

  describe "messagebus object set up correctly" do
    it "has correct headers set for api calls" do
      client = MessagebusApnApi::Messagebus.new(@api_key)
    end
  end

  describe "add cacert file to http communications" do
    it "raises error if cert file does not exist" do
      client = MessagebusApnApi::Messagebus.new(@api_key)
      cert_file_path = File.join(File.dirname(__FILE__), "nofile.pem")
      expect do
        client.cacert_info(cert_file_path)
      end.should raise_error
    end

    it "accepts a cert file that exists" do
      client = MessagebusApnApi::Messagebus.new(@api_key)
      cert_file_path = File.join(File.dirname(__FILE__), "cacert.pem")
      expect do
        client.cacert_info(cert_file_path)
      end.should_not raise_error
    end
  end

  describe "#add_apn_send" do
    it "buffered send that adds to empty buffer" do
      client.add_apn_send(default_apn_send_params, @app_key)
      client.send_flushed?.should be_false
    end
    it "buffered send that adds to empty buffer and sends with flush_buffer flag" do
      expected_request="#{@apn_endpoint}app/#{@app_key}/send"
      FakeWeb.register_uri(:post, expected_request, :body => json_valid_apn_send)
      client.add_apn_send(default_apn_send_params, @app_key, true)
      client.send_flushed?.should be_true
    end
    it "should have user-agent and x-messagebus-key set in request headers" do
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/send", :body => json_valid_apn_send)
      client.add_apn_send(default_apn_send_params, @app_key, true)
      client.send_flushed?.should be_true

      FakeWeb.last_request.get_fields("X-MessageBus-Key").should_not be_nil
      FakeWeb.last_request.get_fields("User-Agent").should_not be_nil
      FakeWeb.last_request.get_fields("Content-Type").should_not be_nil
    end
    it "buffered send that adds to a buffer and auto-flushes" do
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/send", :body => create_success_result(client.send_buffer_size).to_json)
        (client.send_buffer_size-1).times do |idx|
          client.add_apn_send(default_apn_send_params, @app_key)
          client.send_flushed?.should be_false
        end
        client.add_apn_send(default_apn_send_params, @app_key)
        client.send_flushed?.should be_true
        client.results[:results].size.should == client.send_buffer_size

    end
  end

  describe "#apn_app_register" do
    it "buffered register that adds to empty buffer" do
      client.add_apn_register(default_apn_register_params, @app_key)
      client.register_flushed?.should be_false
    end
    it "buffered register that adds to empty buffer and registers with flush_buffer flag" do
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => json_valid_apn_register)
      client.add_apn_register(default_apn_register_params, @app_key, true)
      client.register_flushed?.should be_true
    end
    it "should have user-agent and x-messagebus-key set in request headers" do
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => json_valid_apn_register)
      client.add_apn_register(default_apn_register_params, @app_key, true)
      client.register_flushed?.should be_true

      FakeWeb.last_request.get_fields("X-MessageBus-Key").should_not be_nil
      FakeWeb.last_request.get_fields("User-Agent").should_not be_nil
      FakeWeb.last_request.get_fields("Content-Type").should_not be_nil
    end
    it "buffered register that adds to a buffer and auto-flushes" do
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => create_success_result(client.register_buffer_size).to_json)
        (client.send_buffer_size-1).times do |idx|
          client.add_apn_register(default_apn_register_params, @app_key)
          client.register_flushed?.should be_false
        end
        client.add_apn_register(default_apn_register_params, @app_key)
        client.register_flushed?.should be_true
        client.results[:results].size.should == client.register_buffer_size
    end
  end

  describe "#apn_send_flush" do
    it "apn send flush called on empty buffer" do
      client.apn_send_flush(@app_key)
      client.send_flushed?.should be_false
    end

    it "apn send flush called on partially filled buffer" do
      message_count = 9
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/send", :body => create_success_result(message_count).to_json)
      (message_count).times do |idx|
        client.add_apn_send(default_apn_send_params, @app_key)
        client.send_flushed?.should be_false
      end
      client.apn_send_flush(@app_key)
      client.send_flushed?.should be_true
      client.results[:results].size.should == message_count
    end

    it "doesnt reset connection if under a minute old" do
      current_init_time=client.last_init_time
      current_init_time.should be > Time.now.utc-5
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/send", :body => create_success_result(1).to_json)
      client.add_apn_send(default_apn_send_params, @app_key)
      client.apn_send_flush(@app_key)
      client.send_flushed?.should be_true
      client.results[:results].size.should == 1
      client.last_init_time.should == current_init_time
    end

    it "resets connection if over a minute old" do
      client.last_init_time=Time.now.utc-60
      current_init_time=client.last_init_time
      current_init_time.should be < Time.now.utc-59
      FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/send", :body => create_success_result(1).to_json)
      client.add_apn_send(default_apn_send_params, @app_key)
      client.apn_send_flush(@app_key)
      client.send_flushed?.should be_true
      client.results[:results].size.should == 1
      client.last_init_time.should be > current_init_time
    end
  end

  describe "#send_buffer_size=" do
    it "can set the buffer size" do
      client.send_buffer_size=(10)
      client.send_buffer_size.should == 10
    end

    it "cannot set an invalid buffer size" do
      default_buffer_size = 20
      client.send_buffer_size=(-1)
      client.send_buffer_size.should == default_buffer_size

      client.send_buffer_size=(0)
      client.send_buffer_size.should == default_buffer_size

      client.send_buffer_size=(101)
      client.send_buffer_size.should == default_buffer_size

      client.send_buffer_size=(1)
      client.send_buffer_size.should == 1

      client.send_buffer_size=(100)
      client.send_buffer_size.should == 100
    end
  end

  describe "#apn_register_flush" do
      it "apn register flush called on empty buffer" do
        client.apn_register_flush(@app_key)
        client.register_flushed?.should be_false
      end

      it "apn register flush called on partially filled buffer" do
        message_count = 9
        FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => create_success_result(message_count).to_json)
        (message_count).times do |idx|
          client.add_apn_register(default_apn_register_params, @app_key)
          client.register_flushed?.should be_false
        end
        client.apn_register_flush(@app_key)
        client.register_flushed?.should be_true
        client.results[:results].size.should == message_count
      end

      it "doesnt reset connection if under a minute old" do
        current_init_time=client.last_init_time
        current_init_time.should be > Time.now.utc-5
        FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => create_success_result(1).to_json)
        client.add_apn_register(default_apn_register_params, @app_key)
        client.apn_register_flush(@app_key)
        client.register_flushed?.should be_true
        client.results[:results].size.should == 1
        client.last_init_time.should == current_init_time
      end

      it "resets connection if over a minute old" do
        client.last_init_time=Time.now.utc-60
        current_init_time=client.last_init_time
        current_init_time.should be < Time.now.utc-59
        FakeWeb.register_uri(:post, "#{@apn_endpoint}app/#{@app_key}/register", :body => create_success_result(1).to_json)
        client.add_apn_register(default_apn_register_params, @app_key)
        client.apn_register_flush(@app_key)
        client.register_flushed?.should be_true
        client.results[:results].size.should == 1
        client.last_init_time.should be > current_init_time
      end
    end

    describe "#register_buffer_size=" do
      it "can set the buffer size" do
        client.register_buffer_size=(10)
        client.register_buffer_size.should == 10
      end

      it "cannot set an invalid buffer size" do
        default_buffer_size = 20
        client.register_buffer_size=(-1)
        client.register_buffer_size.should == default_buffer_size

        client.register_buffer_size=(0)
        client.register_buffer_size.should == default_buffer_size

        client.register_buffer_size=(101)
        client.register_buffer_size.should == default_buffer_size

        client.register_buffer_size=(1)
        client.register_buffer_size.should == 1

        client.register_buffer_size=(100)
        client.register_buffer_size.should == 100
      end
    end


  describe "#get_apn_apps" do
    it "can return a list of apps with name and app key" do
      FakeWeb.register_uri(:get, "#{@apn_endpoint}apps", :body => json_valid_apn_apps)
      expect do
        response = client.get_apn_apps
        FakeWeb.last_request.body.should be_nil
        response.should == json_parse(json_valid_apn_apps)
      end.should_not raise_error
    end
  end
end